***********************************************************************
*                                                                     *
* Program......: CMX.PRG                                              *
* System.......: PC POINT-OF-SALE SYSTEM                              *
* Written By...: Ashok Kumar, Infodata Systems Ltd., DUNEDIN, NZ      *
* Date Written.: Nov 1996, May 1998                                   *
* Description..: Database movement to solve index key locking         *
*                COMIX translation functions for XBase++              *
* Revisions....: Lots                                                 *
*                                                                     *
* Functions....: cmxsetscope( 0/1, <condition> )                      *
*                 - behaves like COMIX scopes                         *
*                 - scopes are on current database and INDEX ORDER    *
*                                                                     *
*                cmxclrscope( 0/1 )                                   *
*                 - behaves like COMIX scopes                         *
*                 - clear a given scope, Top or Bottom                *
*                                                                     *
*                cmxclrall( <database> )                              *
*                 - allows you to clear all scopes for a database     *
*                                                                     *
*                AK_Skip( <records> )                                 *
*                 - skip a given number of records                    *
*                 - accounts for FILTERS and SCOPES                   *
*                                                                     *
*                AK_GoTop()                                           *
*                 - go to TOP of record set                           *
*                 - accounts for FILTERS and SCOPES                   *
*                                                                     *
*                AK_GoBot()                                           *
*                 - go to BOTTOM of record set                        *
*                 - accounts for FILTERS and SCOPES                   *
*                                                                     *
* Usage........: Include CMX.CH in all your programs or systemwide .CH*
*                                                                     *
*                Add PUBLIC _cmx_list  := {} to the top of your main  *
*                                                                     *
*                Include AK_Skip(), AK_GoTop and AK_GoBot in BROWSES  *
*                 e.g.  db:gotopblock    := {|| ak_gotop()}           *
*                       db:gobottomblock := {|| ak_gobot()}           *
*                       db:skipblock     := {|n| ak_skip(N)}          *
*                                                                     *
***********************************************************************

Function cmxsetscope(nX, cX)
   LOCAL nfind := 0, nKey
   DEFAULT nX TO 0
   nKey  := pad(UPPER(ALIAS()),10) + str(ordnumber(),2) + str(nX, 1)
   nfind := AScan(_cmx_list, {|elem| nKey = elem[1]})
   if nfind > 0
      if pcount() = 2
         _cmx_list[nfind][2] := cX
      else
         cX := _cmx_list[nfind][2]
      endif
   else
      if pcount() = 2 .and. ordnumber() > 0 .and. .not.empty(Alias())
         AADD(_cmx_list, {nKey, cX} )
      else
         cX := ""          //NIL
      endif
   endif
return cX

Function cmxclrscope(nX)
   LOCAL nfind := 0, nKey
   DEFAULT nX TO 0
   nKey  := pad(UPPER(ALIAS()),10) + str(ordnumber(),2) + str(nX, 1)
   nfind := AScan(_cmx_list, {|elem| nKey = elem[1]})
   if nfind > 0
      ADEL(_cmx_list, nfind)
      ASIZE(_cmx_list, Len(_cmx_list)-1)
   endif
return .t.

Function cmxclrALL(cDbname)
   LOCAL nfind := 0
   DEFAULT cDbname TO Alias()
   FOR nfind = 1 to len(_cmx_list)
      If _cmx_list[nfind][1] = pad(Upper(cDbname), 10)
         ADEL(_cmx_list, nfind)
         ASIZE(_cmx_list, Len(_cmx_list)-1)
      endif
   NEXT
return .t.


Function cmxkeyadd(cX)
return .t.

Function cmxkeydel(cX)
return .t.

Function CMXKEYCOUNT(nX)
return 0


*!*****************************************************************************
*!
*!       Function: AK_SKIP()
*!
*!*****************************************************************************
FUNCTION AK_skip(N)
   LOCAL save_filt := DbFilter()
   LOCAL save_cmx  := cmxsetscope(0)
   LOCAL save_cmxb := cmxsetscope(1)
   LOCAL save_topk := ''
   LOCAL save_botk := ''
   LOCAL save_rec  := recno()
   LOCAL nActSkip := 0

   LOCAL save_key  := ''
   LOCAL eval_cmx, eval_filt

   DEFAULT N to 0

   IF N == 0
      SKIP 0
      RETURN 0
   ENDIF

   if .not.empty(save_cmx)
      save_key := ORDKEY(Ordnumber())
      if ValType(&save_key) = "N"
         ? "What to do with Numeric keys ", save_key, save_cmx ; wait
      else
         save_cmx := "left("+save_key+","+alltrim(str(len(save_cmx)))+") = '" + save_cmx + "'"
      endif
   endif

   * Clear filter, setup while condition and carry on
   SET FILTER TO

   IF .not.(empty(save_filt))
      eval_filt := &( "{||" + save_filt + "}" )
   endif
   IF .not.(empty(save_cmx))
      eval_cmx  := &( "{||" + save_cmx + "}" )
   endif

   *** IF .not.(empty(save_filt) .and. empty(save_cmx))


      IF N > 0 .AND. !EOF()
         // keep skipping until number of skips requested
         // is satisfied
         WHILE nActSkip < N
            DBSKIP(1)

            // end of file
            IF EOF()
               GOTO save_rec          //DBSKIP(-1)
               EXIT
            ENDIF

            // Check filter
            if .not.empty(save_filt) .and. .not.EVAL( eval_filt )
               LOOP
            endif

            // Evaluate condition to each record
            IF .not.empty(save_cmx) .and. .not.EVAL( eval_cmx )
               GOTO save_rec          //DBSKIP(-1)
               EXIT
            ENDIF

            // Mantain counter
            nActSkip++
            save_rec := recno()

         ENDDO

      ELSEIF N < 0
         WHILE nActSkip > N
            DBSKIP(-1)
            // This check is important! BOF bangs do not
            // sit on phanton record. Rather the top record
            // is evaluated, and may meet the condition
            IF BOF()
               EXIT
            ENDIF

            // Check filter
            if .not.empty(save_filt) .and. .not.EVAL( eval_filt )
               LOOP
            endif

            // Evaluate condition to each record
            IF .not.empty(save_cmx) .and. .not.EVAL( eval_cmx )
               GOTO save_rec             //DBSKIP(1)
               EXIT
            ENDIF

            // Mantain counter
            nActSkip--
            save_rec := recno()

         ENDDO

      ENDIF

   *** else
   ***    DbSkip(N)
   ***    nActSkip := N
   *** endif
   SET FILTER TO &save_filt
   DbSkip(0)
   RETURN (nActSkip)


FUNCTION AK_gotop()
   LOCAL save_filt := DbFilter()
   LOCAL save_cmxt := cmxsetscope(0)
   LOCAL save_cmxb := cmxsetscope(1)
   LOCAL save_topk := ''
   LOCAL save_botk := ''
   LOCAL eval_file

   SET FILTER TO

   if .not.empty(save_cmxt)
      save_key := ORDKEY(Ordnumber())
      dbSeek(save_cmxt, .T.)
      save_trec := recno()
   else
      DbGoTop()
      save_trec := recno()
   endif

   if .not.empty(save_filt)
      eval_file = &( "{||" + save_filt + "}" )
      DO WHILE recno() <> save_trec
         IF .NOT.EVAL( eval_file )
            EXIT
         endif
         dbSkip(-1)
      ENDDO
   endif
   SET FILTER TO &save_filt
RETURN nil


FUNCTION AK_gobot()
   LOCAL save_filt := DbFilter()
   LOCAL save_cmxt := cmxsetscope(0)
   LOCAL save_cmxb := cmxsetscope(1)
   LOCAL save_topk := ''
   LOCAL save_botk := ''
   LOCAL eval_file

   SET FILTER TO

   if .not.empty(save_cmxt)
      save_key := ORDKEY(Ordnumber())
      tkey := Left(save_cmxb, Len(save_cmxb)-1) + CHR(ASC(right(save_cmxb,1)) + 1)
      dbSeek(tkey, .T.)
      save_brec := recno()
   else
      DbGoBottom()
      save_brec := recno()
   endif

   if .not.empty(save_filt)
      eval_file = &( "{||" + save_filt + "}" )
      DO WHILE recno() <> save_brec
         IF .NOT.EVAL( eval_filt )
            EXIT
         endif
         dbSkip(1)
      ENDDO
   endif

   SET FILTER TO &save_filt
RETURN nil

***********************************************************************

